
package net.javainthebox.flickrviewer;

import javafx.scene.CustomNode;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Polygon;
import javafx.scene.shape.Rectangle;

// Op` TlC̃y[WɎgp
class Triangle extends CustomNode {
    var x: Integer;
    var y: Integer;
    var width: Integer;
    var height: Integer;
    var isUpward: Boolean;
    var action: function(): Void;
 
    override var onMouseClicked = function(event: MouseEvent) {
        if (not disable) {
            action();
        }
    }

    override function create(): Node {
        Group {
            content: [
                Rectangle {
                    x: x
                    y: y
                    width: width,
                    height: height
                    stroke: Color.web("#666666")
                    fill: bind if (not disable) {
                        if (pressed) {
                            Color.GRAY
                        } else if (hover) {
                            Color.web("#666666")
                        } else {
                            Color.BLACK;
                        }
                    } else {
                        Color.BLACK;
                    }
                },
                if (isUpward) {
                    Polygon {
                        points: [
                            x + width/2, y + height*.2,
                            x + width/2 + height*.3, y + height*.8,
                            x + width/2 - height*.3, y + height*.8
                        ]
                        fill: bind if (disable) Color.GRAY else Color.WHITE
                    }
                } else {
                    Polygon {
                        points: [
                            x + width/2, y + height*.8,
                            x + width/2 + height*.3, y + height*.2,
                            x + width/2 - height*.3, y + height*.2
                        ]
                        fill: bind if (disable) Color.GRAY else Color.WHITE
                    }
                }
            ]
        }
    }
}

// TlC\yC
public class ThumbnailPane extends CustomNode {
    // TlC̑IԂύXR[R[obN֐
    public-init var update: function(title: String, url: String);

    // TlC
    // TlČɉāAy[W̃tOZbg
    public var thumbnails: Thumbnail[] on replace {
        if (calcIndex(page) < sizeof thumbnails) {
            forwardRemainsFlag = true;
        }
        for (thumbnail in thumbnails) {
            thumbnail.action = thumbnailSelected;
        }
        updateVisibleThumbnails();
    }
 
    // ݕ\ĂTlC
    var visibleThumbnails: Thumbnail[];
    // IĂTlC
    var selectedThumbnail: Thumbnail;
    // TlC̃y[W
    var page: Integer = 0;
    // y[W肪ł邩ǂtO
    var forwardRemainsFlag = false;
    var backwardRemainsFlag = false;

    def numPerVertical = 5;
    def thumbnailSize = 120;
    public-read def width: Integer = 120;
    public-read def height: Integer = 660;
    def arrowHeight = 25;

    // TlC̑IԂύXR[֐
    function thumbnailSelected(thumbnail: Thumbnail): Void {
        if (selectedThumbnail != null) {
            selectedThumbnail.selected = false;
        }
        selectedThumbnail = thumbnail;
        update(selectedThumbnail.title, selectedThumbnail.imageUrl);
    }
 
    // y[W肳ꂽꍇɕ\XV֐
    function updateVisibleThumbnails() {
        visibleThumbnails = for (y in [0.. numPerVertical - 1]) {
            var index = page * numPerVertical + y;
            var thumbnail = thumbnails[index];
            thumbnail.translateX = 5;
            thumbnail.translateY = y * thumbnailSize + 35;
            thumbnail;
        }
    }

    function calcIndex(page: Integer): Integer {
        (page + 1) * numPerVertical;
    }

    public override function create(): Node {
        Group{
            content: bind [
                Rectangle {
                    x: 0 y:0
                    width: width height: height
                    fill: null
                    stroke: Color.web("#666666")
                }
                Triangle {
                    x: 0 y: 0
                    width: width height: arrowHeight
                    isUpward: true
                    disable: bind not backwardRemainsFlag
                    action: function(): Void {
                        page--;
                        if (page == 0) {
                            backwardRemainsFlag = false;
                        }
                        if (forwardRemainsFlag == false
                            and calcIndex(page) < sizeof thumbnails) {
                            forwardRemainsFlag = true;
                        }
                        updateVisibleThumbnails();
                    }
                }
                visibleThumbnails,
                Triangle {
                    x: 0 y: height - arrowHeight
                    width: width height: arrowHeight
                    isUpward: false
                    disable: bind not forwardRemainsFlag
                    action: function(): Void {
                        page++;
                        if (calcIndex(page) >= sizeof thumbnails) {
                            forwardRemainsFlag = false;
                        }
                        if (backwardRemainsFlag == false) {
                            backwardRemainsFlag = true;
                        }
                        updateVisibleThumbnails();
                    }
                }
            ]
        }
    }
}